#
# Time Drive - based on duplicity - Encrypted bandwidth efficient backup.
#
from PyQt4 import QtCore
import threading

class BackgroundTask(QtCore.QThread, object):
	"""
	BackgroundTask class: base class for threads used in time-drive
	"""
	
	_restart = None

	def __init__(self, parent = None):
		"""
		Initialization function for BackgroundTask
		
		@type parent: QThread Object
		@param parent:
		"""
		super(BackgroundTask, self).__init__()
		self.parent = parent
		self.settings = None

	def taskComplete(self, taskName, userMessage):
		"""
		emits a message when a run is complete need to call manually
		
		@type taskName: String
		@param taskName: the task who sents the message
		@type userMessage: String
		@param userMessage: the message you want to show the user 
		"""
		self.emit(QtCore.SIGNAL("taskComplete(QString, QString)"), 
			QtCore.QString(taskName), QtCore.QString(userMessage))
	
	def taskError(self, taskName, error, userMessage):
		"""
		emits a message when a task has an error need to call manually
		
		@type taskName: String
		@param taskName: the task who sents the message
		@type error: String
		@param error: type of error as string
		@type userMessage : String
		@param userMessage: the message you want to show the user 
		"""
		self.emit(QtCore.SIGNAL("taskError(QString, QString, QString)"), 
			QtCore.QString(taskName), QtCore.QString(error), QtCore.QString(userMessage))
	
	def setupTaskProgress(self, maximum, message = None):
		"""
		emits a message with task progress need to call manually
		
		@type maximum: int
		@param maximum: the maximum of elements to process
		@type message: String
		@param message: default=None, informative message or task name 
		"""
		self.emit(QtCore.SIGNAL("setupTaskProgress(int, QString)"),
			maximum, QtCore.QString(message))
	
	def updateTaskProgress(self, progress):
		"""
		emits a int to update the progress bar
		
		@type progress: int
		@param progress:
		"""
		self.emit(QtCore.SIGNAL("updateTaskProgress(int)"), progress)
	
	def toggleControls(self, option):
		"""
		emits a boolean to indicate if the controls need to be toggled in the ui
		
		@type option: Boolean
		@param option:
		"""
		self.emit(QtCore.SIGNAL("toggleControls(bool)"), option)

	def questionPrompt(self, title, question):
		"""
		emits a question and passes on the thread that fire this method
		
		@type title: String
		@param title:
		@type question: String
		@param question:
		"""
		self.cond = threading.Condition()
		self.emit(QtCore.SIGNAL("questionPrompt(QString,QString,PyQt_PyObject)"),title,question,self.cond)
		self.cond.acquire()		
		self.cond.wait()
		self.cond.release()

	def _set_restart(self, value):
		"""
		restart status
		
		@type value: Boolean
		@param value: Indication if restart is needed 
		"""
		self._restart = value

	def _get_restart(self):
		"""
		restart status
		
		@rtype: Boolean
		"""
		return self._restart

	restart = property(_get_restart, _set_restart)
