#
# Time Drive - based on duplicity - Encrypted bandwidth efficient backup.
#
from timedrive.utils import duplicity_interface
import backgroundtask

class RestoreFiles(backgroundtask.BackgroundTask):
	"""Background class that restores files from the remote backup."""
	
	def __init__(self, parent = None):
		super(RestoreFiles, self).__init__(parent)
		self.exiting = False
		self.destination = None
		self.RestoreList = None
	
	def StartRestore(self, RestoreList, settings, destination = None):
		self.settings = settings
		self.RestoreList = RestoreList

		if destination is not None or self.settings.Advanced_DefaultRestoreEnabled == True:
			if destination is None:
				self.destination = self.settings.Advanced_DefaultRestoreLocation
			else:
				self.destination = destination
			self.start()
		else:
			self.destination = None
			self.RestoreList = None
		
	
	def run(self):
		"""Main thread method.  Toggles restore options on/off and then 
		restores the files."""
		
		S3_AccessId_Settings = None
		S3_SecretKey_Settings = None
		S3_BucketType_Settings = None
		
		#Setup ProgressBar
		numItems = len(self.RestoreList)
		self.setupTaskProgress(numItems, "Restoring Files ... ")
		progress = 0		
		
		# Set Amazon S3 Secret Key and Access Id
		if self.settings.AmazonS3Backup == True:
			S3_AccessId_Settings = self.settings.AmazonS3_AccessId
			S3_SecretKey_Settings = self.settings.AmazonS3_SecretKey
		
		if self.settings.AmazonS3_Location == "European":
			S3_BucketType_Settings = self.settings.AmazonS3_Location

		# Main Restore Operation
		for restoreFile in self.RestoreList:
			progress = progress + 1
			relativePath, archiveUrl, gnuPassphrase, restoreTime, S3_LoginInfo_Folder = self.RestoreList[restoreFile]
			
			# Restore from the most recent snapshot
			if (restoreTime == 'Most Current')|(restoreTime == 'Most Recent'):
				
				# Use global settings for Amazon S3
				if S3_LoginInfo_Folder == '':
					duplicity_interface.restore_file(relativePath, archiveUrl, 
						gnuPassphrase, self.destination, None, 
						S3_AccessId_Settings, S3_SecretKey_Settings, 
						S3_BucketType_Settings)
				
				# Use either remote or folder specific settings for Amazon S3
				else:
					S3_AccessId_Folder, S3_SecretKey_Folder = S3_LoginInfo_Folder.split()
					duplicity_interface.restore_file(relativePath, archiveUrl,
						gnuPassphrase, self.destination, None,
						S3_AccessId_Folder, S3_SecretKey_Folder, 
						S3_BucketType_Settings)
			
			# Restore from a previous snapshot
			else:
				duplicity_interface.restore_file(relativePath, archiveUrl, 
					gnuPassphrase, self.destination, restoreTime, 
					S3_AccessId_Settings, S3_SecretKey_Settings, 
					S3_BucketType_Settings)
			
			self.updateTaskProgress(progress)
		
		self.updateTaskProgress(numItems)
		self.taskComplete(_("Restore Files"), _("Files Restored Successfully"))

