#!/usr/bin/env python
#
# Time Drive - based on duplicity - Encrypted bandwidth efficient backup.
#
# Copyright 2009 Rob Oakes	<LyX-Devel@oak-tree>

import os
import time
import re
import sys
import socket
import log
from timedrive.backupsettings import includeitem

MANUAL = 0
DAY = 1
WEEK = 2
MONTH = 3

from duplicity import dup_time

def handle_error(err_message):
	"""
	handles the error produced by duplicity and tries to make a proper response
	
	@type err_message: String
	@param err_message: the error message from duplicity
	
	@rtype: int
	@return: returns -1 if the error can't be process. return 0 if backendException
		return 1 to 99 if fatal error occured
	"""
	if "Fatal Error" in err_message:
		print "found Fatal Error"
		if " --allow-source-mismatch" in err_message:
			return 1
		else:
			return 99
	elif "BackendException" in err_message:
		print "found BackendException"
		return 0
	else:
		return -1

def find_file(file_name):
	"""
	Using the file_name, search in the common places. Return the 
	path for the file or None if the file couldn't be found.
	
	@type file_name: String
	@param file_name: the filename to search
	
	@rtype: String
	@return: if found the full path with filename, if not found returns None
	"""
	# Ordered by priority, keep global paths before local paths
	
	current_dir = os.path.abspath(os.path.dirname(__file__))
	common_paths = [os.path.join(sys.prefix, 'local', 'share', 'pixmaps'),
			os.path.join(sys.prefix, 'share', 'time-drive'),
		os.path.join(sys.prefix, "share", "pixmaps"),
		"/usr/local/bin", "/usr/bin", 
		os.path.abspath(os.path.join(current_dir, "../..")),
		os.path.abspath(os.path.join(current_dir, "../../ui")),
		os.path.abspath(os.path.join(current_dir, "../../ui", "Resources"))]
	
	for path in common_paths:
		filename = os.path.join(path, file_name)
		if os.access(filename, os.F_OK):
			return filename

	return None

def time_String2Int(time_string):
	"""
	Convenience funciton for converting raw time strings to integer format.
	
	@type time_string: String
	@param time_string: time as string
	
	@rtype: Int
	@return: return the time
	"""
	time_int = dup_time.stringtotime(time_string)
	return time_int


def time_GetCurrentTime():
	"""Convenience function which can retrieve the current time"""
	intNow = time.time()
	strNow = time_Int2String(intNow)
	return strNow


def time_Int2String(time_int):
	"""Convenience function for converting time in integer format to raw string."""
	time_string = dup_time.timetostring(time_int)
	return time_string


def time_String2Pretty(time_string):
	"""Convenience funciton for converting raw time strings to a more readable format."""
	time_pretty = dup_time.stringtopretty(time_string)
	return time_pretty


def time_Int2Pretty(time_int):
	"""Convenience function for converting time integers to a more readable format."""
	time_pretty = dup_time.timetopretty(time_int)
	return time_pretty


def verify_ExcludePattern(exclude_pattern):
	"""Verifies the specified exclude_pattern or path to ensure that it doesn't contain
	illegal characters."""
	pattern = str(exclude_pattern[0].toUtf8()).strip()
	
	if len(pattern) == 0:
		return None
	
	if pattern.find(':') >=0:
		raise Exception(_("Exclude patterns can\'t contain \':\' char !"))
		return None
	
	return pattern

def isValidRegExp(regexp):
	try:
		re.compile(str(regexp))
		return True
	except re.error:
		print "This RE is invalid"
		return False

def setup_cron(backup_frequency):
	"""Method that creates a listing in the users crontab file so that backups can run 
	unattended.  If using Mac OS X, crontab must be enabled and the user must have a valid
	cron file before running."""
	
	programName = find_file('timedrive-backup')
	os.system("crontab -l | grep -v timedrive-backup | crontab - ")
	
	cron_line = ''
	
	if backup_frequency == MANUAL:	# Manual Backups
		cron_line = ''
	elif backup_frequency == DAY:
		cron_line = 'echo "@daily {cmd}"'
	elif backup_frequency == WEEK:
		cron_line = 'echo "@weekly {cmd}"'
	elif backup_frequency == MONTH:
		cron_line = 'echo "@monthly {cmd}"'
	
	if len(cron_line) > 0:
		cmd = programName + ' > /dev/null 2>&1'
		cmd = 'nice -n 19 ' + cmd
		cron_line = cron_line.replace('{cmd}', cmd)
		os.system("(crontab -l; %s) | crontab -" % cron_line)

def checkServer(host, port):
	serverSocket = socket.socket()
	serverSocket.settimeout(0.25)
	try:
		serverSocket.connect((str(host), int(port)))
		return True
	except socket.error:
		return False

def hostnameChk(host):
	try:
		socket.gethostbyname(host)
		return True
	except socket.gaierror,e:
		return False


def ipFormatChk(ip_str):
	pattern = r"\b(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\.(25[0-5]|2[0-4][0-9]|[01]?[0-9][0-9]?)\b"
	if re.match(pattern, ip_str):
		return True
	else:
		return False
	
def findFolder(IncludeList, folderName, folderType = None):
	for f in IncludeList:
		if folderName == f.FolderName and (folderType is None or folderType == f.BackupType):
			return f

def getPortProtocol(proto):
	"""
	Convert protocol name to port
	
	@type proto: String
	@param proto: is the name of the protocol
	
	@rtype: int
	@return: returns the port of a protocol
	"""
	if proto == "ssh":
		return 22
	elif proto == "ftp":
		return 21
	elif proto == "webdav":
		return 80
	elif proto == "webdavs":
		return 443
	elif proto == "rsync":
		return 873
	else:
		log.Error("undefined protocol: " + str(proto))
		return 00

def backupTypeToText(includeItem):
	"""
	convert folder backup type to text
	Note: when text change method textToBackupType
	
	@type includeItem: IncludeItem Object
	@param includeItem:
	"""
	if includeItem.BackupType == includeItem.GENERALSETTINGS:
		return _("General Settings")
	elif includeItem.BackupType == includeItem.LOCAL:
		return _("Folder: Local")
	elif includeItem.BackupType == includeItem.REMOTE:
		return _("Folder: Remote")
		

def textToBackupType(text):
	"""
	convert text to folder backup type
	Note: when text change method backupTypeToText
	
	@type text: String
	@param text:
	"""
	if text == _("General Settings"):
		return includeitem.IncludeItem().GENERALSETTINGS
	elif text == _("Folder: Local"):
		return includeitem.IncludeItem().LOCAL
	elif text == _("Folder: Remote"):
		return includeitem.IncludeItem().REMOTE